#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time    : 2024/8/7 8:54
# @Author  : wangqinggang
# @Email   : wqg1993@qq.com
# @File    : subscriber.py
import cv2
import zmq
import av
import os
import sys

class DxrSubscriber:
    def __init__(self, zmq_url, codec='h264', mode='r', result=True):
        self.zmq_url = zmq_url
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.SUB)
        self.socket.setsockopt(zmq.RCVHWM, 1000)  # High Water Mark
        self.socket.connect(self.zmq_url)
        self.socket.setsockopt_string(zmq.SUBSCRIBE, "")  # Subscribe to all messages
        self.decoder = av.codec.CodecContext.create(codec, mode)
        self.decoder.options = {'strict': 'experimental'}
        self.null_fd = os.open(os.devnull, os.O_WRONLY)
        self.old_stderr = os.dup(sys.stderr.fileno())
        self.result = result
        os.dup2(self.null_fd, sys.stderr.fileno())

    def pull(self):
        try:
            if self.result:
                topic, data = self.socket.recv_multipart()
            else:
                topic, data = self.socket.recv()
            packet = av.packet.Packet(data)
            if packet.size < 1000:
                return None, None
            try:
                frames = self.decoder.decode(packet)
            except Exception:
                frames = []
            for frame in frames:
                img = frame.to_ndarray(format='bgr24')
                return img, topic
        except zmq.ZMQError:
            pass
        except Exception:
            pass
        return None, None

    def close(self):
        self.socket.close()
        os.dup2(self.old_stderr, sys.stderr.fileno())
        os.close(self.null_fd)
        os.close(self.old_stderr)

    def show(self, img, port):
        cv2.namedWindow(f'received {port}', cv2.WINDOW_NORMAL)
        cv2.imshow(f'received {port}', img)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            exit()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time    : 2024/8/7 8:54
# @Author  : wangqinggang
# @Email   : wqg1993@qq.com
# @File    : __init__.py
__version__ = "0.0.0"
from .subscriber import DxrSubscriber



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time    : 2024/8/7 8:55
# @Author  : wangqinggang
# @Email   : wqg1993@qq.com
# @File    : setup.py
import setuptools
import os
import re

# Read the version from the package
def read_version():
    init_file_path = os.path.join(os.path.dirname(__file__), "dxrsubscriber", "__init__.py")
    with open(init_file_path, "r", encoding="utf-8") as fp:
        for line in fp:
            match = re.match(r"^__version__ = ['\"]([^'\"]*)['\"]", line)
            if match:
                return match.group(1)
    raise RuntimeError("Unable to find version string.")
setuptools.setup(
    name="dxrsubscriber",
    version=read_version(),
    author="Your Name",
    author_email="your.email@example.com",
    description="A ZMQ subscriber for receiving and decoding video streams.",
    long_description=open('README.md', encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/zmq_subscriber",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "pyzmq",
        "av",
        "opencv-python==4.5.1.48"
    ],
)
